@@ -1,23 +1,38 @@
-// Copyright (C) 2006-2007 David Sugar, Tycho Softworks.
+// Copyright (C) 2006-2008 David Sugar, Tycho Softworks.
 //
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 3 of the License, or
+// This file is part of GNU ucommon.
+//
+// GNU ucommon is free software: you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as published 
+// by the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// This program is distributed in the hope that it will be useful,
+// GNU ucommon is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
-// GNU General Public License for more details.
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
 //
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+// You should have received a copy of the GNU Lesser General Public License
+// along with GNU ucommon.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <config.h>
 #include <ucommon/thread.h>
 #include <ucommon/fsys.h>
 #include <ucommon/string.h>
+
+#ifdef HAVE_LINUX_VERSION_H
+#include <linux/version.h>
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,8)
+#ifdef	HAVE_POSIX_FADVISE
+#undef	HAVE_POSIX_FADVISE
+#endif
+#endif
+#endif
+
+#ifndef	_XOPEN_SOURCE
+#define	_XOPEN_SOURCE 600
+#endif
+
 #include <stdio.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -37,7 +52,7 @@ const fsys::offset_t fsys::end = (size_t)(-1);
 
 #ifdef	_MSWINDOWS_
 
-int remapError(void)
+int fsys::remapError(void)
 {
 	DWORD err = GetLastError();
 
@@ -224,12 +239,16 @@ void fsys::open(const char *path, access_t access)
 	bool append = false;
 	DWORD amode;
 	DWORD smode = 0;
-	DWORD attr = FILE_ATTRIBUTE_NORMAL | FILE_FLAG_RANDOM_ACCESS;
+	DWORD attr = FILE_ATTRIBUTE_NORMAL;
 
 	close();
 
 	switch(access)
 	{
+	case ACCESS_STREAM:
+#ifdef	FILE_FLAG_SEQUENTIAL_SCAN
+		attr |= FILE_FLAG_SEQUENTIAL_SCAN; 
+#endif
 	case ACCESS_RDONLY:
 		amode = GENERIC_READ;
 		smode = FILE_SHARE_READ;
@@ -237,6 +256,8 @@ void fsys::open(const char *path, access_t access)
 	case ACCESS_WRONLY:
 		amode = GENERIC_WRITE;
 		break;
+	case ACCESS_RANDOM:
+		attr |= FILE_FLAG_RANDOM_ACCESS;
 	case ACCESS_REWRITE:
 		amode = GENERIC_READ | GENERIC_WRITE;
 		smode = FILE_SHARE_READ;
@@ -282,7 +303,7 @@ void fsys::create(const char *path, access_t access, unsigned mode)
 	DWORD amode;
 	DWORD cmode;
 	DWORD smode = 0;
-	DWORD attr = FILE_ATTRIBUTE_NORMAL | FILE_FLAG_RANDOM_ACCESS;
+	DWORD attr = FILE_ATTRIBUTE_NORMAL; 
 	unsigned flags = 0;
 	switch(access)
 	{
@@ -291,11 +312,14 @@ void fsys::create(const char *path, access_t access, unsigned mode)
 		cmode = OPEN_ALWAYS;
 		smode = FILE_SHARE_READ;
 		break;
+	case ACCESS_STREAM:
 	case ACCESS_WRONLY:
 		amode = GENERIC_WRITE;
 		cmode = CREATE_ALWAYS; 
 		flags = O_WRONLY | O_CREAT | O_TRUNC;
 		break;
+	case ACCESS_RANDOM:
+		attr |= FILE_FLAG_RANDOM_ACCESS;
 	case ACCESS_REWRITE:
 		amode = GENERIC_READ | GENERIC_WRITE;
 		cmode = OPEN_ALWAYS;
@@ -377,6 +401,10 @@ void fsys::operator=(const fsys& from)
 	}
 }
 
+void fsys::drop(offset_t size)
+{
+}
+
 void fsys::seek(offset_t pos)
 {
 	DWORD rpos = pos;
@@ -392,9 +420,6 @@ void fsys::seek(offset_t pos)
 
 #else
 
-inline int remapError(void)
-	{return errno;};
-
 ssize_t fsys::read(void *buf, size_t len)
 {
 	if(ptr) {
@@ -463,9 +488,11 @@ void fsys::create(const char *path, access_t access, unsigned mode)
 	case ACCESS_RDONLY:
 		flags = O_RDONLY | O_CREAT;
 		break;
+	case ACCESS_STREAM:
 	case ACCESS_WRONLY:
 		flags = O_WRONLY | O_CREAT | O_TRUNC;
 		break;
+	case ACCESS_RANDOM:
 	case ACCESS_SHARED:
 	case ACCESS_REWRITE:
 		flags = O_RDWR | O_CREAT;
@@ -481,6 +508,12 @@ void fsys::create(const char *path, access_t access, unsigned mode)
 	fd = ::open(path, flags, mode);
 	if(fd == INVALID_HANDLE_VALUE)
 		error = remapError();
+#ifdef HAVE_POSIX_FADVISE
+	else {
+		if(access == ACCESS_RANDOM)
+			posix_fadvise(fd, (off_t)0, (off_t)0, POSIX_FADV_RANDOM);
+	}
+#endif
 }
 
 int fsys::createDir(const char *path, unsigned mode)
@@ -505,12 +538,18 @@ void fsys::open(const char *path, access_t access)
 
 	switch(access)
 	{
+	case ACCESS_STREAM:
+#if defined(O_STREAMING)
+		flags = O_RDONLY | O_STREAMING;
+		break;
+#endif
 	case ACCESS_RDONLY:
 		flags = O_RDONLY;
 		break;
 	case ACCESS_WRONLY:
 		flags = O_WRONLY;
 		break;
+	case ACCESS_RANDOM:
 	case ACCESS_SHARED:
 	case ACCESS_REWRITE:
 		flags = O_RDWR;
@@ -527,6 +566,15 @@ void fsys::open(const char *path, access_t access)
 	fd = ::open(path, flags);
 	if(fd == INVALID_HANDLE_VALUE)
 		error = remapError();
+#ifdef HAVE_POSIX_FADVISE
+	else {
+		// linux kernel bug prevents use of POSIX_FADV_NOREUSE in streaming...
+		if(access == ACCESS_STREAM)
+			posix_fadvise(fd, (off_t)0, (off_t)0, POSIX_FADV_SEQUENTIAL);
+		else if(access == ACCESS_RANDOM)
+			posix_fadvise(fd, (off_t)0, (off_t)0, POSIX_FADV_RANDOM);
+	}
+#endif
 }
 
 int fsys::stat(const char *path, struct stat *ino)
@@ -602,6 +650,13 @@ void fsys::operator=(const fsys& from)
 	}
 }
 
+void fsys::drop(offset_t size)
+{
+#ifdef	HAVE_POSIX_FADVISE
+	posix_fadvise(fd, (off_t)0, size, POSIX_FADV_DONTNEED);
+#endif
+}
+
 void fsys::seek(offset_t pos)
 {
 	unsigned long rpos = pos;
