@@ -167,8 +167,10 @@ void keyfile::load(const char *path)
 
 	for(;;) {
 		*lp = 0;
-		fgets(lp, size, fp);
-		String::chop(lp, "\r\n\t ");
+		if(NULL == fgets(lp, size, fp))
+			lp[0] = 0;
+		else
+			String::chop(lp, "\r\n\t ");
 		ep = lp + strlen(lp);
 		if(ep != lp) {
 			--ep;
